@@ -24,8 +24,15 @@
 #include <ctype.h>
 #include "version.h"
 #include "admutil_pvt.h"
+#include "libadminutil/admutil-int.h"
 #include "libadminutil/distadm.h"
-
+#include <ssl.h>
+#if defined(USE_OPENLDAP)
+#include <lber.h>
+#else
+#include <ldappr.h>
+#include <ldap_ssl.h>
+#endif
 #ifdef XP_WIN32
 #define strcasecmp stricmp
 #define strncasecmp _strnicmp
@@ -96,14 +103,19 @@ find_file_in_paths(
     return retval;
 }
 
-/* Copy from libadmin.....  */
-static unsigned char uuset[] = {
-'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T',
-'U','V','W','X','Y','Z','a','b','c','d','e','f','g','h','i','j','k','l','m','n',
-'o','p','q','r','s','t','u','v','w','x','y','z','0','1','2','3','4','5','6','7',
-'8','9','+','/' };
- 
-static int LDAP_CALL LDAP_CALLBACK
+#if defined(USE_OPENLDAP)
+static int
+admin_ldap_rebind_proc(
+	LDAP *ld, LDAP_CONST char *url,
+	ber_tag_t request, ber_int_t msgid,
+	void *arg)
+{
+  AdmldapHdnlPtr ptr = (AdmldapHdnlPtr)arg;
+
+  return admutil_ldap_bind(ld, ptr->sieDN, ptr->passwd, LDAP_SASL_SIMPLE, NULL, NULL, NULL, NULL);
+}
+#else /* NOT USE_OPENLDAP */
+static int
 admin_ldap_rebind_proc (LDAP *ld, char **whop, char **passwdp,
                        int *authmethodp, int freeit, void *arg) {
   AdmldapHdnlPtr ptr = (AdmldapHdnlPtr)arg;
@@ -116,58 +128,7 @@ admin_ldap_rebind_proc (LDAP *ld, char **whop, char **passwdp,
 
   return LDAP_SUCCESS;
 }
-
-PR_IMPLEMENT(int) 
-admutil_uuencode(unsigned char *src, unsigned char *dst, int srclen)
-{
-   int  i, r;
-   unsigned char *p;
- 
-/* To uuencode, we snip 8 bits from 3 bytes and store them as
-   6 bits in 4 bytes.   6*4 == 8*3 (get it?) and 6 bits per byte
-   yields nice clean bytes
- 
-   It goes like this:
-        AAAAAAAA BBBBBBBB CCCCCCCC
-        turns into the standard set of uuencode ascii chars indexed by numbers:
-        00AAAAAA 00AABBBB 00BBBBCC 00CCCCCC
- 
-        Snip-n-shift, snip-n-shift, etc....
- */
- 
-   for (p=dst,i=0; i < srclen; i += 3) {
-     /* Do 3 bytes of src */
-     register char b0, b1, b2;
- 
-     b0 = src[0];
-     if (i==srclen-1)
-       b1 = b2 = '\0';
-     else if (i==srclen-2) {
-       b1 = src[1];
-       b2 = '\0';
-     }
-     else {
-       b1 = src[1];
-       b2 = src[2];
-     }
- 
-     *p++ = uuset[b0>>2];
-     *p++ = uuset[(((b0 & 0x03) << 4) | ((b1 & 0xf0) >> 4))];
-     *p++ = uuset[(((b1 & 0x0f) << 2) | ((b2 & 0xc0) >> 6))];
-     *p++ = uuset[b2 & 0x3f];
-     src += 3;
-   }
-   *p = 0;      /* terminate the string */
-   r = (unsigned char *)p - (unsigned char *)dst;/* remember how many we did */
- 
-   /* Always do 4-for-3, but if not round threesome, have to go
-          clean up the last extra bytes */
- 
-   for( ; i != srclen; i--)
-                *--p = '=';
- 
-   return r;
-}
+#endif
 
 PR_IMPLEMENT(char**)
 admutil_strsdup(char** orig)
@@ -189,6 +150,27 @@ admutil_strsdup(char** orig)
   return dest;
 }
 
+PR_IMPLEMENT(char**)
+admutil_strsdup_berval(struct berval** orig)
+{
+  int  cnt = 0, i = 0;
+  char **dest;
+  struct berval **tmpptr;
+  
+  if (!orig) return NULL;
+
+  tmpptr = orig;
+  while (*tmpptr++ != NULL) cnt++;
+
+  dest = (char**)PR_Malloc((cnt+1)*sizeof(char*));
+
+  for (i=0; i < cnt; i++) dest[i] = PL_strndup(orig[i]->bv_val, orig[i]->bv_len);
+
+  dest[cnt] = NULL;
+
+  return dest;
+}
+
 PR_IMPLEMENT(void)
 admutil_strsFree(char** target)
 {
@@ -385,8 +367,8 @@ dn2AttrName(char* dn, char* rootDN)
     attrLen--;
   }
 
-  ldap_value_free(dnList);
-  ldap_value_free(rootList);
+  admutil_strsFree(dnList);
+  admutil_strsFree(rootList);
 
   return PL_strdup(buf);
 }
@@ -548,9 +530,11 @@ listDelete(ListNodePtr list, char* name) {
   /* Not implemented */
   ListNodePtr   node, nextptr;
 
-  if (!list) return NULL;
+  if (!list || !name) {
+    goto done;
+  }
 
-  if (!strcasecmp(list->name, name)) {
+  if (list->name && !strcasecmp(list->name, name)) {
     nextptr = list->next;
     if (list->name) PR_Free(list->name);
     if (list->dflag) deleteValue ((ValueType)list->val);
@@ -560,7 +544,7 @@ listDelete(ListNodePtr list, char* name) {
 
   node = list;
   while (node->next) {
-    if (!strcasecmp(node->next->name, name)) {
+    if (node->next->name && !strcasecmp(node->next->name, name)) {
       nextptr = node->next->next;
       if (node->next->name) PR_Free (node->next->name);
       if (node->next->dflag) deleteValue ((ValueType)node->next->val);
@@ -570,6 +554,8 @@ listDelete(ListNodePtr list, char* name) {
     }
     node = node->next;
   }
+
+done:
   return list;
 }
 
@@ -729,6 +715,18 @@ treeFindNode(TreeNodePtr node, char* name)
   else return NULL;
 }
 
+static int
+treeHasNode(AdmldapInfo info, char* name)
+{
+  int result = 0;
+  AdmldapHdnlPtr admInfo = (AdmldapHdnlPtr)info;
+
+  if (admInfo && admInfo->configInfo) {
+    result = treeFindNode(admInfo->configInfo, name) ? 1 : 0;
+  }
+  return result;
+}
+
 ValueType
 treeFindValue(TreeNodePtr root, char* name)
 {
@@ -1185,21 +1183,26 @@ admldapBuildInfoOnly(char* configRoot, int *errorcode)
   configInfo = treeImport(fileStream, errorcode);
   fclose(fileStream);
 
-  if (!configInfo) { *errorcode = ADMUTIL_SYSTEM_ERR; return NULL; }
+  if (!configInfo) {
+    *errorcode = ADMUTIL_SYSTEM_ERR;
+    PR_smprintf_free(path);
+    return NULL;
+  }
 
   admInfo = (AdmldapHdnlPtr)PR_Malloc(sizeof(AdmldapHdnl));
-  if (!admInfo) { *errorcode = ADMUTIL_SYSTEM_ERR; return NULL; }
+  if (!admInfo) {
+    *errorcode = ADMUTIL_SYSTEM_ERR;
+    treeRemoveTree(configInfo);
+    PR_smprintf_free(path);
+    return NULL;
+  }
   memset(admInfo, '\0', sizeof(AdmldapHdnl));
 
   admInfo->configFilePath = path; /* hand off memory */
   path = NULL;
-  if (!admInfo->configFilePath) {
-      PR_Free(admInfo);
-      *errorcode = ADMUTIL_SYSTEM_ERR;
-      return NULL;
-  }
 
-  admInfo->configInfo = configInfo;
+  admInfo->configInfo = configInfo; /* hand off memory */
+  configInfo = NULL;
 
   if (!(admInfo->serverDirectoryURL = treeFindValueAt(admInfo->configInfo, "ldapurl", 0))) { /* admInfo owns malloced memory now */
     /* Error open file  */
@@ -1208,8 +1211,9 @@ admldapBuildInfoOnly(char* configRoot, int *errorcode)
     return NULL;
   }
 
-  if (ldap_url_parse(admInfo->serverDirectoryURL, &ldapInfo)) {
+  if (admutil_ldap_url_parse(admInfo->serverDirectoryURL, &ldapInfo, 0, &admInfo->secure)) {
     *errorcode = ADMUTIL_SYSTEM_ERR;
+    destroyAdmldap((AdmldapInfo)admInfo);
     return NULL;
   }
 
@@ -1247,21 +1251,24 @@ admldapBuildInfoOnly(char* configRoot, int *errorcode)
     /* EOF */
   default:
     password = strchr(buf, ':');
-    *password++ = '\0';
-    while (*password) {
-      if (*password == ' ') password++;
-      else break;
-    }
+    if (password) {
+      *password++ = '\0';
+      while (*password) {
+        if (*password == ' ') password++;
+        else break;
+      }
     
-    name = buf;
-    if (*password) {
-      *errorcode =  ADMUTIL_OP_OK;
-      admInfo->admpwFilePath = path; /* hand off memory */
-      path = NULL;
-      admInfo->localAdminName = PL_strdup(name);
-      admInfo->localAdminPassword = PL_strdup(password);
-    }
-    else {
+      name = buf;
+      if (*password) {
+        *errorcode =  ADMUTIL_OP_OK;
+        admInfo->admpwFilePath = path; /* hand off memory */
+        path = NULL;
+        admInfo->localAdminName = PL_strdup(name);
+        admInfo->localAdminPassword = PL_strdup(password);
+      } else {
+        *errorcode =  ADMUTIL_OP_FAIL;
+      }
+    } else {
       *errorcode =  ADMUTIL_OP_FAIL;
     }
   }
@@ -1269,6 +1276,10 @@ admldapBuildInfoOnly(char* configRoot, int *errorcode)
 
   PR_smprintf_free(path);
 
+  if (!treeHasNode((AdmldapInfo)admInfo, "securitydir")) {
+    admldapSetSecurityDir((AdmldapInfo)admInfo, configRoot);
+  }
+
   return (AdmldapInfo)admInfo;
 }
 
@@ -1305,7 +1316,8 @@ admldapBuildInfoCbk(char* configRoot, char *(*cbk)(), int *errorcode)
   }
 
   if (!secureLDAP) {
-    admInfo->ldapHndl = ldap_init(admInfo->ldapInfo->lud_host, admInfo->ldapInfo->lud_port);
+    admInfo->ldapHndl = admutil_ldap_init(admInfo, NULL, admInfo->ldapInfo->lud_host,
+                                          admInfo->ldapInfo->lud_port, 0, 0, NULL);
   }
 
   /* authenticate to LDAP server*/
@@ -1338,7 +1350,7 @@ admldapBuildInfoCbk(char* configRoot, char *(*cbk)(), int *errorcode)
         }
 
         if (!secureLDAP) {
-          ldapError = ldap_simple_bind_s(admInfo->ldapHndl, admInfo->sieDN, siePasswd);
+          ldapError = admutil_ldap_bind(admInfo->ldapHndl, admInfo->sieDN, siePasswd, LDAP_SASL_SIMPLE, NULL, NULL, NULL, NULL);
           if (ldapError == LDAP_SUCCESS) break;
 
           /* Quit on errors other than password problems */
@@ -1349,14 +1361,14 @@ admldapBuildInfoCbk(char* configRoot, char *(*cbk)(), int *errorcode)
 
   if ((ldapError != LDAP_SUCCESS ) && !(secureLDAP)){
 #ifdef LDAP_DEBUG
-      ldap_perror(admInfo->ldapHndl, "ldap_simple_bind_s");
+    fprintf(stderr, "admutil_ldap_bind: [%s] error [%d:%s]\n", admInfo->sieDN, ldapError, ldap_err2string(ldapError));
 #endif
       switch (ldapError) {
       case LDAP_INAPPROPRIATE_AUTH:
       case LDAP_INVALID_CREDENTIALS:
       case LDAP_INSUFFICIENT_ACCESS:
         /* authenticate failed: Should not continue */
-        ldap_unbind(admInfo->ldapHndl);
+        ldap_unbind_ext(admInfo->ldapHndl, NULL, NULL);
         admInfo->ldapHndl = NULL;
         *errorcode = ADMUTIL_LDAP_ERR;
         break;;
@@ -1364,12 +1376,12 @@ admldapBuildInfoCbk(char* configRoot, char *(*cbk)(), int *errorcode)
       case LDAP_ALIAS_PROBLEM:
       case LDAP_INVALID_DN_SYNTAX:
         /* Not a good user DN */
-        ldap_unbind(admInfo->ldapHndl);
+        ldap_unbind_ext(admInfo->ldapHndl, NULL, NULL);
         admInfo->ldapHndl = NULL;
         *errorcode = ADMUTIL_LDAP_ERR;
         break;
       default:
-        ldap_unbind(admInfo->ldapHndl);
+        ldap_unbind_ext(admInfo->ldapHndl, NULL, NULL);
         admInfo->ldapHndl = NULL;
         *errorcode = ADMUTIL_LDAP_ERR;
       }
@@ -1452,7 +1464,7 @@ destroyAdmldap(AdmldapInfo info)
       admInfo->passwd = NULL;
     }
     if (admInfo->ldapHndl) {
-      ldap_unbind(admInfo->ldapHndl);
+      ldap_unbind_ext(admInfo->ldapHndl, NULL, NULL);
       admInfo->ldapHndl = NULL;
     }
     
@@ -1487,7 +1499,7 @@ admldapGetSecurity(AdmldapInfo info)
   AdmldapHdnlPtr admInfo = (AdmldapHdnlPtr)info;
   int            ldapSecurity;
 
-  if (admInfo->ldapInfo->lud_options & LDAP_URL_OPT_SECURE) ldapSecurity = 1;
+  if (admInfo->secure) ldapSecurity = 1;
   else ldapSecurity = 0;
 
   return ldapSecurity;
@@ -1514,8 +1526,7 @@ admldapGetSecurityDir(AdmldapInfo info)
   char          *securitydir;
 
   securitydir = treeFindValueAt(admInfo->configInfo, "securitydir", 0);
-  if (!securitydir) return NULL;
-  else return securitydir;
+  return securitydir;
 }
 
 PR_IMPLEMENT(int)
@@ -1540,8 +1551,7 @@ admldapGetSIEDN(AdmldapInfo info)
   char          *ldapSIEDN = NULL;
 
   ldapSIEDN = treeFindValueAt(admInfo->configInfo, "sie", 0);
-  if (!ldapSIEDN) return NULL;
-  else return ldapSIEDN;
+  return ldapSIEDN;
 }
 
 PR_IMPLEMENT(int)
@@ -1583,8 +1593,7 @@ admldapGetISIEDN(AdmldapInfo info)
   char          *ldapISIEDN = NULL;
 
   ldapISIEDN = treeFindValueAt(admInfo->configInfo, "isie", 0);
-  if (!ldapISIEDN) return NULL;
-  else  return ldapISIEDN;
+  return ldapISIEDN;
 
 }
 
@@ -1618,8 +1627,9 @@ admldapSetDirectoryURL(AdmldapInfo info, const char *ldapurl)
   LDAPURLDesc    *ldapInfo;
   int errorcode = ADMUTIL_OP_OK;
   int removeFlag = 0;
+  int secure = 0;
 
-  if (!ldapurl || ldap_url_parse(ldapurl, &ldapInfo)) {
+  if (!ldapurl || admutil_ldap_url_parse(ldapurl, &ldapInfo, 0, &secure)) {
 	/* if the given url is not valid, don't do anything, just return an error */
     errorcode = ADMUTIL_SYSTEM_ERR;
     goto done;
@@ -1640,6 +1650,7 @@ admldapSetDirectoryURL(AdmldapInfo info, const char *ldapurl)
   /* set the new values */
   admInfo->serverDirectoryURL = PL_strdup(ldapurl);
   admInfo->ldapInfo = ldapInfo;
+  admInfo->secure = secure;
   treeAddNameValue(admInfo->configInfo, "ldapurl", (char *)ldapurl);
 
 done:
@@ -1651,7 +1662,7 @@ admldapSetLDAPHndl(AdmldapInfo info, LDAP *ld)
 {
   AdmldapHdnlPtr admInfo = (AdmldapHdnlPtr)info;
 
-  if (admInfo->ldapHndl) ldap_unbind(admInfo->ldapHndl);
+  if (admInfo->ldapHndl) ldap_unbind_ext(admInfo->ldapHndl, NULL, NULL);
   admInfo->ldapHndl = ld;
 }
 
@@ -1670,8 +1681,7 @@ admldapGetSysUser(AdmldapInfo info)
   char          *sysuser = NULL;
 
   sysuser = treeFindValueAt(admInfo->configInfo, "sysuser", 0);
-  if (!sysuser) return NULL;
-  else  return sysuser;
+  return sysuser;
 
 }
 
@@ -1682,8 +1692,7 @@ admldapGetSysGroup(AdmldapInfo info)
   char          *sysgroup = NULL;
 
   sysgroup = treeFindValueAt(admInfo->configInfo, "sysgroup", 0);
-  if (!sysgroup) return NULL;
-  else  return sysgroup;
+  return sysgroup;
 
 }
 
@@ -1694,8 +1703,7 @@ admldapGetAdminDomain(AdmldapInfo info)
   char          *admindomain = NULL;
 
   admindomain = treeFindValueAt(admInfo->configInfo, "AdminDomain", 0);
-  if (!admindomain) return NULL;
-  else  return admindomain;
+  return admindomain;
 
 }
 
@@ -1706,8 +1714,7 @@ admldapGetExpressRefreshRate(AdmldapInfo info)
   char          *expressrefreshrate = NULL;
 
   expressrefreshrate = treeFindValueAt(admInfo->configInfo, "ExpressRefreshRate", 0);
-  if (!expressrefreshrate) return NULL;
-  else  return expressrefreshrate;
+  return expressrefreshrate;
 
 }
 
@@ -1718,8 +1725,7 @@ admldapGetExpressCGITimeout(AdmldapInfo info)
   char          *expresscgitimeout = NULL;
 
   expresscgitimeout = treeFindValueAt(admInfo->configInfo, "ExpressCGITimeout", 0);
-  if (!expresscgitimeout) return NULL;
-  else  return expresscgitimeout;
+  return expresscgitimeout;
 
 }
 
@@ -1730,8 +1736,7 @@ admldapGetLdapStart(AdmldapInfo info)
   char          *ldapstart = NULL;
 
   ldapstart = treeFindValueAt(admInfo->configInfo, "ldapStart", 0);
-  if (!ldapstart) return NULL;
-  else  return ldapstart;
+  return ldapstart;
 
 }
 
@@ -1787,8 +1792,8 @@ admldapGetUserDN(AdmldapInfo info, char *uid)
 	goto done;
   }
   uidFilter = PR_smprintf("(uid=%s)", uid?uid:admInfo->localAdminName);
-  err = ldap_search_s(ld, baseDN, LDAP_SCOPE_SUBTREE, uidFilter,
-				      NULL, 0, &result);
+  err = ldap_search_ext_s(ld, baseDN, LDAP_SCOPE_SUBTREE, uidFilter,
+                          NULL, 0, NULL, NULL, NULL, -1, &result);
   if (err != LDAP_SUCCESS || ldap_count_entries(ld, result) == 0) {
     ldap_msgfree(result);
 	goto done;
@@ -1896,3 +1901,685 @@ admGetCachedSIEPWD()
 
   return result;
 }
+
+#if defined(USE_OPENLDAP)
+/* mozldap ldap_init and ldap_url_parse accept a hostname in the form
+   host1[:port1]SPACEhost2[:port2]SPACEhostN[:portN]
+   where SPACE is a single space (0x20) character
+   for openldap, we have to convert this to a string like this:
+   PROTO://host1[:port1]/SPACEPROTO://host2[:port2]/SPACEPROTO://hostN[:portN]/
+   where PROTO is ldap or ldaps or ldapi
+   if proto is NULL, assume hostname_or_uri is really a valid ldap uri
+*/
+static char *
+convert_to_openldap_uri(const char *hostname_or_uri, int port, const char *proto)
+{
+  char *retstr = NULL;
+  char *my_copy = NULL;
+  char *start = NULL;
+  char *iter = NULL;
+  char *s = NULL;
+  const char *brkstr = " ";
+
+  if (!hostname_or_uri) {
+	return NULL;
+  }
+
+  my_copy = PL_strdup(hostname_or_uri);
+  /* see if hostname_or_uri is an ldap uri */
+  if (!proto && !PL_strncasecmp(my_copy, "ldap", 4)) {
+    start = my_copy + 4;
+    if ((*start == 's') || (*start == 'i')) {
+      start++;
+    }
+    if (!PL_strncmp(start, "://", 3)) {
+      *start = '\0';
+      proto = my_copy;
+      start += 3;
+    } else {
+#ifdef DEBUG
+      fprintf(stderr, "convert_to_openldap_uri: The given LDAP URI [%s] is not valid\n", hostname_or_uri);
+#endif
+      goto end;
+    }
+  } else if (!proto) {
+#ifdef DEBUG
+    fprintf(stderr, "convert_to_openldap_uri: The given LDAP URI [%s] is not valid\n", hostname_or_uri);
+#endif
+    goto end;
+  } else {
+    start = my_copy; /* just assume it's not a uri */
+  }
+	    
+  for (s = strtok_r(my_copy, brkstr, &iter); s != NULL;
+       s = strtok_r(NULL, brkstr, &iter)) {
+	char *ptr;
+	int last = 0;
+	/* strtok will grab the '/' at the end of the uri, if any,
+	   so terminate parsing there */
+	if ((ptr = strchr(s, '/'))) {
+      *ptr = '\0';
+      last = 1;
+	}
+	if (retstr) {
+      retstr = PR_sprintf_append(retstr, "/ %s://%s", proto, s);
+	} else {
+      retstr = PR_smprintf("%s://%s", proto, s);
+	}
+	if (last) {
+      break;
+	}
+  }
+
+  /* add the port on the last one */
+  retstr = PR_sprintf_append(retstr, ":%d/", port);
+end:
+  PL_strfree(my_copy);
+  return retstr;    
+}
+#endif /* USE_OPENLDAP */
+
+const char *
+admutil_urlparse_err2string( int err )
+{
+  const char *s="internal error";
+
+  switch( err ) {
+  case 0:
+	s = "no error";
+	break;
+  case LDAP_URL_ERR_BADSCOPE:
+	s = "invalid search scope";
+	break;
+  case LDAP_URL_ERR_MEM:
+	s = "unable to allocate memory";
+	break;
+  case LDAP_URL_ERR_PARAM:
+	s = "bad parameter to an LDAP URL function";
+	break;
+#if defined(USE_OPENLDAP)
+  case LDAP_URL_ERR_BADSCHEME:
+	s = "does not begin with ldap://, ldaps://, or ldapi://";
+	break;
+  case LDAP_URL_ERR_BADENCLOSURE:
+	s = "missing trailing '>' in enclosure";
+	break;
+  case LDAP_URL_ERR_BADURL:
+	s = "not a valid LDAP URL";
+	break;
+  case LDAP_URL_ERR_BADHOST:
+	s = "hostname part of url is not valid or not given";
+	break;
+  case LDAP_URL_ERR_BADATTRS:
+	s = "attribute list not formatted correctly or missing";
+	break;
+  case LDAP_URL_ERR_BADFILTER:
+	s = "search filter not correct";
+	break;
+  case LDAP_URL_ERR_BADEXTS:
+	s = "extensions not specified correctly";
+	break;
+#else /* !USE_OPENLDAP */
+  case LDAP_URL_ERR_NOTLDAP:
+	s = "missing ldap:// or ldaps:// or ldapi://";
+	break;
+  case LDAP_URL_ERR_NODN:
+	s = "missing suffix";
+	break;
+#endif
+  }
+
+  return( s );
+}
+
+/* there are various differences among url parsers - directory server
+   needs the ability to parse partial URLs - those with no dn - and
+   needs to be able to tell if it is a secure url (ldaps) or not */
+int
+admutil_ldap_url_parse(const char *url, LDAPURLDesc **ludpp, int require_dn, int *secure)
+{
+  PR_ASSERT(url);
+  PR_ASSERT(ludpp);
+  int rc;
+  const char *url_to_use = url;
+#if defined(USE_OPENLDAP)
+  char *urlescaped = NULL;
+#endif
+
+  if (secure) {
+    *secure = 0;
+  }
+#if defined(USE_OPENLDAP)
+  /* openldap does not support the non-standard multi host:port URLs supported
+     by mozldap - so we have to fake out openldap - replace all spaces with %20 -
+     replace all but the last colon with %3A
+     Go to the 3rd '/' or to the end of the string (convert only the host:port part) */
+  if (url) {
+	char *p = strstr(url, "://");
+	if (p) {
+      int foundspace = 0;
+      int coloncount = 0;
+      char *lastcolon = NULL;
+      p += 3;
+      for (; *p && (*p != '/'); p++) {
+		if (*p == ' ') {
+          foundspace = 1;
+		}
+		if (*p == ':') {
+          coloncount++;
+          lastcolon = p;
+		}
+      }
+      if (foundspace) {
+		char *src = NULL, *dest = NULL;
+		/* have to convert url */
+		/* len * 3 is way too much, but acceptable */
+		urlescaped = PR_Calloc(strlen(url) * 3, sizeof(char));
+		dest = urlescaped;
+		/* copy the scheme */
+        src = strstr(url, "://");
+		src += 3;
+		memcpy(dest, url, src-url);
+		dest += (src-url);
+		/* we have to convert all spaces to %20 - we have to convert
+		   all colons except the last one to %3A */
+		for (; *src; ++src) {
+          if (src < p) {
+			if (*src == ' ') {
+              memcpy(dest, "%20", 3);
+              dest += 3;
+			} else if ((coloncount > 1) && (*src == ':') && (src != lastcolon)) {
+              memcpy(dest, "%3A", 3);
+              dest += 3;
+			} else {
+              *dest++ = *src;
+			}
+          } else {
+			*dest++ = *src;
+          }
+		}
+		*dest = '\0';
+		url_to_use = urlescaped;
+      }
+	}
+  }
+#endif
+
+#if defined(HAVE_LDAP_URL_PARSE_NO_DEFAULTS)
+  rc = ldap_url_parse_no_defaults(url_to_use, ludpp, require_dn);
+  if (!rc && *ludpp && secure) {
+    *secure = (*ludpp)->lud_options & LDAP_URL_OPT_SECURE;
+  }
+#else /* openldap */
+#if defined(HAVE_LDAP_URL_PARSE_EXT) && defined(LDAP_PVT_URL_PARSE_NONE) && defined(LDAP_PVT_URL_PARSE_NOEMPTY_DN)
+  rc = ldap_url_parse_ext(url_to_use, ludpp, require_dn ? LDAP_PVT_URL_PARSE_NONE : LDAP_PVT_URL_PARSE_NOEMPTY_DN);
+#else
+  rc = ldap_url_parse(url_to_use, ludpp);
+  if ((rc || !*ludpp) && !require_dn) { /* failed - see if failure was due to missing dn */
+    size_t len = strlen(url_to_use);
+    /* assume the url is just scheme://host:port[/] - add the empty string
+       as the DN (adding a trailing / first if needed) and try to parse
+       again
+    */
+    char *urlcopy = PR_smprintf("%s%s%s", url_to_use, (url_to_use[len-1] == '/' ? "" : "/"), "");
+    if (*ludpp) {
+      ldap_free_urldesc(*ludpp); /* free the old one, if any */
+    }
+    rc = ldap_url_parse(urlcopy, ludpp);
+    PL_strfree(urlcopy);
+    urlcopy = NULL;
+    if (0 == rc) { /* only problem was the DN - free it */
+      PL_strfree((*ludpp)->lud_dn);
+      (*ludpp)->lud_dn = NULL;
+    }
+  }
+#endif
+  if (!rc && *ludpp && secure) {
+    *secure = (*ludpp)->lud_scheme && !strcmp((*ludpp)->lud_scheme, "ldaps");
+  }
+#endif /* openldap */
+
+#if defined(USE_OPENLDAP)
+  if (urlescaped && (*ludpp) && (*ludpp)->lud_host) {
+	/* have to unescape lud_host - can unescape in place */
+	char *p = strstr((*ludpp)->lud_host, "://");
+	if (p) {
+      char *dest = NULL;
+      p += 3;
+      dest = p;
+      /* up to the first '/', unescape the host */
+      for (; *p && (*p != '/'); p++) {
+		if (!strncmp(p, "%20", 3)) {
+          *dest++ = ' ';
+          p += 2;
+		} else if (!strncmp(p, "%3A", 3)) {
+          *dest++ = ':';
+          p += 2;
+		} else {
+          *dest++ = *p;
+		}
+      }
+      /* just copy the remainder of the host, if any */
+      while (*p) {
+		*dest++ = *p++;
+      }
+      *dest = '\0';
+	}
+  }
+  PL_strfree(urlescaped);
+#endif
+  return rc;
+}
+
+/*
+  Perform LDAP init and return an LDAP* handle.  If ldapurl is given,
+  that is used as the basis for the protocol, host, port, and whether
+  to use starttls (given on the end as ldap://..../?????starttlsOID
+  If hostname is given, LDAP or LDAPS is assumed, and this will override
+  the hostname from the ldapurl, if any.  If port is > 0, this is the
+  port number to use.  It will override the port in the ldapurl, if any.
+  If no port is given in port or ldapurl, the default will be used based
+  on the secure setting (389 for ldap, 636 for ldaps)
+  secure takes 1 of 2 values - 0 means regular ldap, 1 means ldaps
+  filename is the ldapi file name - if this is given, and no other options
+  are given, ldapi is assumed.
+ */
+LDAP *
+admutil_ldap_init(
+  AdmldapInfo info,
+  const char *ldapurl, /* full ldap url */
+  const char *hostname, /* can also use this to override
+                           host in url */
+  int port, /* can also use this to override port in url */
+  int secure, /* 0 for ldap, 1 for ldaps */
+  int shared, /* if true, LDAP* will be shared among multiple threads */
+  const char *filename /* for ldapi */
+)
+{
+  LDAPURLDesc	*ludp = NULL;
+  LDAP *ld = NULL;
+  int rc = 0;
+  int secureurl = 0;
+  int ldap_version3 = LDAP_VERSION3;
+
+  /* if ldapurl is given, parse it */
+  if (ldapurl && ((rc = admutil_ldap_url_parse(ldapurl, &ludp, 0, &secureurl)) ||
+                  !ludp)) {
+#ifdef DEBUG
+	fprintf(stderr, "admutil_ldap_init: Could not parse given LDAP URL [%s] : error [%s]\n",
+                    ldapurl, /* ldapurl cannot be NULL here */
+                    admutil_urlparse_err2string(rc));
+#endif
+	goto done;
+  }
+
+  /* use url host if no host given */
+  if (!hostname && ludp && ludp->lud_host) {
+	hostname = ludp->lud_host;
+  }
+
+  /* use url port if no port given */
+  if (!port && ludp && ludp->lud_port) {
+	port = ludp->lud_port;
+  }
+
+  /* use secure setting from url if none given */
+  if (!secure && ludp) {
+	if (secureurl) {
+      secure = 1;
+	}
+  }
+
+#if defined(USE_OPENLDAP)
+  if (ldapurl) {
+	rc = ldap_initialize(&ld, ldapurl);
+	if (rc) {
+#ifdef DEBUG
+      fprintf(stderr, "admutil_ldap_init: Could not initialize LDAP connection to [%s]: %d:%s\n",
+                      ldapurl, rc, ldap_err2string(rc));
+#endif
+      goto done;
+	}
+  } else {
+	char *makeurl = NULL;
+	if (filename) {
+      makeurl = PR_smprintf("ldapi://%s/", filename);
+	} else { /* host port */
+      makeurl = convert_to_openldap_uri(hostname, port, (secure == 1 ? "ldaps" : "ldap"));
+	}
+	rc = ldap_initialize(&ld, makeurl);
+	if (rc) {
+#ifdef DEBUG
+      fprintf(stderr, "admutil_ldap_init: Could not initialize LDAP connection to [%s]: %d:%s\n",
+                      makeurl, rc, ldap_err2string(rc));
+#endif
+      PL_strfree(makeurl);
+      makeurl = NULL;
+      goto done;
+	}
+    PL_strfree(makeurl);
+    makeurl = NULL;
+  }
+#else /* !USE_OPENLDAP */
+  if (filename) {
+	/* ldapi in mozldap client is not yet supported */
+  } else if (secure == 1) {
+	ld = ldapssl_init(hostname, port, secure);
+  } else { /* regular ldap and/or starttls */
+	/*
+	 * Leverage the libprldap layer to take care of all the NSPR
+	 * integration.
+	 * Note that ldapssl_init() uses libprldap implicitly.
+	 */
+	ld = prldap_init(hostname, port, shared);
+  }
+#endif /* !USE_OPENLDAP */
+
+  /* must explicitly set version to 3 */
+  ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, &ldap_version3);
+
+  if ((ld != NULL) && !filename) {
+	/*
+	 * Set SSL strength (server certificate validity checking).
+	 */
+	if (secure > 0) {
+#if defined(USE_OPENLDAP)
+      char *certdir = admldapGetSecurityDir(info);
+      int optval = 0;
+#endif /* !USE_OPENLDAP */
+      int ssl_strength = 0;
+      LDAP *myld = NULL;
+
+      /* we can only use the set functions below with a real
+         LDAP* if it has already gone through ldapssl_init -
+         so, use NULL if using starttls */
+      if (secure == 1) {
+		myld = ld;
+      }
+
+		/* verify certificate only */
+#if defined(USE_OPENLDAP)
+      ssl_strength = LDAP_OPT_X_TLS_NEVER;
+#else /* !USE_OPENLDAP */
+      ssl_strength = LDAPSSL_AUTH_CERT;
+#endif /* !USE_OPENLDAP */
+
+#if defined(USE_OPENLDAP)
+      if ((rc = ldap_set_option(ld, LDAP_OPT_X_TLS_REQUIRE_CERT, &ssl_strength))) {
+#ifdef DEBUG
+		fprint(stderr, "admutil_ldap_init: "
+                        "failed: unable to set REQUIRE_CERT option to %d\n", ssl_strength);
+#endif
+      }
+      /* tell it where our cert db is */
+      if ((rc = ldap_set_option(ld, LDAP_OPT_X_TLS_CACERTDIR, certdir))) {
+#ifdef DEBUG
+		fprintf(stderr, "admutil_ldap_init: "
+                        "failed: unable to set CACERTDIR option to %s\n", certdir);
+#endif
+      }
+      PL_strfree(certdir);
+      certdir = NULL;
+#if defined(LDAP_OPT_X_TLS_PROTOCOL_MIN)
+      optval = LDAP_OPT_X_TLS_PROTOCOL_SSL3;
+      if ((rc = ldap_set_option(ld, LDAP_OPT_X_TLS_PROTOCOL_MIN, &optval))) {
+#ifdef DEBUG
+		fprintf(stderr, "admutil_ldap_init: "
+                        "failed: unable to set minimum TLS protocol level to SSL3\n");
+#endif
+      }
+#endif /* LDAP_OPT_X_TLS_PROTOCOL_MIN */
+      if ((rc = ldap_set_option(ld, LDAP_OPT_X_TLS_NEWCTX, &optval))) {
+#ifdef DEBUG
+		fprintf(stderr, "admutil_ldap_init: "
+                        "failed: unable to create new TLS context\n");
+#endif
+      }
+#else  /* !USE_OPENLDAP */
+      if ((rc = ldapssl_set_strength(myld, ssl_strength)) ||
+          (rc = ldapssl_set_option(myld, SSL_ENABLE_SSL2, PR_FALSE)) ||
+          (rc = ldapssl_set_option(myld, SSL_ENABLE_SSL3, PR_TRUE)) ||
+          (rc = ldapssl_set_option(myld, SSL_ENABLE_TLS, PR_TRUE))) {
+		int prerr = PR_GetError();
+
+#ifdef DEBUG
+		fprintf(stderr, "admutil_ldap_init: "
+                        "failed: unable to set SSL options ("
+                        "error %d - %s)\n",
+                         prerr, PR_ErrorToString(prerr, PR_LANGUAGE_I_DEFAULT));
+#endif
+      }
+      if (secure == 1) {
+		/* tell bind code we are using SSL */
+		ldap_set_option(ld, LDAP_OPT_SSL, LDAP_OPT_ON);
+      }
+#endif /* !USE_OPENLDAP */
+	}
+  }
+
+#ifdef DEBUG
+  fprintf(stderr, "admutil_ldap_init: "
+                  "Success: set up conn to [%s:%d]%s\n",
+                  hostname, port,
+                  secure ? " using TLS/SSL" : "");
+#endif
+done:
+  ldap_free_urldesc(ludp);
+
+  return( ld );
+}
+
+static int
+admutil_ldap_get_lderrno(LDAP *ld, char **m, char **s)
+{
+  int rc = LDAP_SUCCESS;
+
+#if defined(USE_OPENLDAP)
+  ldap_get_option(ld, LDAP_OPT_RESULT_CODE, &rc);
+  if (m) {
+    ldap_get_option(ld, LDAP_OPT_MATCHED_DN, m);
+  }
+  if (s) {
+#ifdef LDAP_OPT_DIAGNOSTIC_MESSAGE
+    ldap_get_option(ld, LDAP_OPT_DIAGNOSTIC_MESSAGE, s);
+#else
+    ldap_get_option(ld, LDAP_OPT_ERROR_STRING, s);
+#endif
+  }
+#else /* !USE_OPENLDAP */
+  rc = ldap_get_lderrno( ld, m, s );
+#endif
+  return rc;
+}
+
+#ifndef LDAP_SASL_EXTERNAL
+#define LDAP_SASL_EXTERNAL      "EXTERNAL"      /* TLS/SSL extension */
+#endif
+
+/*
+ * Does the correct bind operation simple/sasl/cert depending
+ * on the arguments passed in.
+ */
+int
+admutil_ldap_bind(
+    LDAP *ld, /* ldap connection */
+    const char *bindid, /* usually a bind DN for simple bind */
+    const char *creds, /* usually a password for simple bind */
+    const char *mech, /* name of mechanism */
+    LDAPControl **serverctrls, /* additional controls to send */
+    LDAPControl ***returnedctrls, /* returned controls */
+    struct timeval *timeout, /* timeout */
+    int *msgidp /* pass in non-NULL for async handling */
+)
+{
+  int rc = LDAP_SUCCESS;
+  int secure = 0;
+  struct berval bvcreds = {0, NULL};
+  LDAPMessage *result = NULL;
+  struct berval *servercredp = NULL;
+#if defined(USE_OPENLDAP)
+  /* openldap doesn't have a SSL/TLS yes/no flag - so grab the
+     ldapurl, parse it, and see if it is a secure one */
+  char *ldapurl = NULL;
+
+  ldap_get_option(ld, LDAP_OPT_URI, &ldapurl);
+  if (ldapurl && !PL_strncasecmp(ldapurl, "ldaps", 5)) {
+    secure = 1;
+  }
+  PL_strfree(ldapurl);
+  ldapurl = NULL;
+#else /* !USE_OPENLDAP */
+  ldap_get_option(ld, LDAP_OPT_SSL, &secure);
+#endif
+
+#ifdef EXTERNAL_AUTH_SUPPORTED
+  if (secure && mech && !strcmp(mech, LDAP_SASL_EXTERNAL)) {
+	/* SSL connections will use the server's security context
+	   and cert for client auth */
+	rc = slapd_SSL_client_auth(ld);
+
+	if (rc != 0) {
+#ifdef DEBUG
+      fprintf(stderr, "admutil_ldap_bind: "
+                      "Error: could not configure the server for cert "
+                      "auth - error %d - make sure the server is "
+                      "correctly configured for SSL/TLS\n", rc);
+#endif
+      goto done;
+	} else {
+#ifdef DEBUG
+      fprintf(stderr, "admutil_ldap_bind: "
+                      "Set up conn to use client auth\n");
+#endif
+    }
+	bvcreds.bv_val = NULL; /* ignore username and passed in creds */
+	bvcreds.bv_len = 0; /* for external auth */
+	bindid = NULL;
+  } else { /* other type of auth */
+#endif /* EXTERNAL_AUTH_SUPPORTED */
+	bvcreds.bv_val = (char *)creds;
+	bvcreds.bv_len = creds ? strlen(creds) : 0;
+#ifdef EXTERNAL_AUTH_SUPPORTED
+  }
+#endif /* EXTERNAL_AUTH_SUPPORTED */
+
+  /* The connection has been set up - now do the actual bind, depending on
+     the mechanism and arguments */
+  if (!mech || (mech == LDAP_SASL_SIMPLE) ||
+      !strcmp(mech, LDAP_SASL_EXTERNAL)) {
+	int mymsgid = 0;
+#ifdef DEBUG
+	fprintf(stderr, "admutil_ldap_bind: "
+                    "attempting %s bind with id [%s] creds [%s]\n",
+                    mech ? mech : "SIMPLE",
+                    bindid, creds);
+#endif
+	if ((rc = ldap_sasl_bind(ld, bindid, mech, &bvcreds, serverctrls,
+                             NULL /* clientctrls */, &mymsgid))) {
+#ifdef DEBUG
+      fprintf(stderr, "admutil_ldap_bind: "
+                      "Error: could not send bind request for id "
+                      "[%s] mech [%s]: error %d (%s) %d (%s) %d (%s)\n",
+                      bindid ? bindid : "(anon)",
+                      mech ? mech : "SIMPLE",
+                      rc, ldap_err2string(rc),
+                      PR_GetError(), PR_ErrorToString(PR_GetError(), PR_LANGUAGE_I_DEFAULT),
+                      errno, strerror(errno));
+#endif
+      goto done;
+	}
+
+	if (msgidp) { /* let caller process result */
+      *msgidp = mymsgid;
+	} else { /* process results */
+      rc = ldap_result(ld, mymsgid, LDAP_MSG_ALL, timeout, &result);
+      if (-1 == rc) { /* error */
+		rc = admutil_ldap_get_lderrno(ld, NULL, NULL);
+#ifdef DEBUG
+		fprintf(stderr, "admutil_ldap_bind: "
+                        "Error reading bind response for id "
+                        "[%s] mech [%s]: error %d (%s)\n",
+                        bindid ? bindid : "(anon)",
+                        mech ? mech : "SIMPLE",
+                        rc, ldap_err2string(rc));
+#endif
+		goto done;
+      } else if (rc == 0) { /* timeout */
+		rc = LDAP_TIMEOUT;
+#ifdef DEBUG
+		fprintf(stderr, "admutil_ldap_bind: "
+                        "Error: timeout after [%ld.%ld] seconds reading "
+                        "bind response for [%s] mech [%s]\n",
+                        timeout ? timeout->tv_sec : 0,
+                        timeout ? timeout->tv_usec : 0,
+                        bindid ? bindid : "(anon)",
+                        mech ? mech : "SIMPLE");
+#endif
+		goto done;
+      }
+      /* if we got here, we were able to read success result */
+      /* Get the controls sent by the server if requested */
+      if (returnedctrls) {
+        if ((rc = ldap_parse_result(ld, result, &rc, NULL, NULL,
+                                    NULL, returnedctrls,
+                                    0)) != LDAP_SUCCESS) {
+#ifdef DEBUG
+          fprintf(stderr, "admutil_ldap_bind: "
+                          "Error: could not bind id "
+                          "[%s] mech [%s]: error %d (%s)\n",
+                          bindid ? bindid : "(anon)",
+                          mech ? mech : "SIMPLE",
+                          rc, ldap_err2string(rc));
+#endif
+          goto done;
+		}
+      }
+
+      /* parse the bind result and get the ldap error code */
+      if ((rc = ldap_parse_sasl_bind_result(ld, result, &servercredp,
+                                            0))) {
+		rc = admutil_ldap_get_lderrno(ld, NULL, NULL);
+#ifdef DEBUG
+        fprintf(stderr, "admutil_ldap_bind: "
+                        "Error: could not read bind results for id "
+                        "[%s] mech [%s]: error %d (%s)\n",
+                        bindid ? bindid : "(anon)",
+                        mech ? mech : "SIMPLE",
+                        rc, ldap_err2string(rc));
+#endif
+		goto done;
+      }
+	}
+  } else {
+    rc = -1;
+#ifdef SASL_AUTH_SUPPORTED
+	/* a SASL mech */
+	rc = slapd_ldap_sasl_interactive_bind(ld, bindid, creds, mech,
+                                          serverctrls, returnedctrls,
+                                          msgidp);
+	if (LDAP_SUCCESS != rc) {
+#ifdef DEBUG
+      fprintf(stderr, "admutil_ldap_bind: "
+                      "Error: could not perform interactive bind for id "
+                      "[%s] mech [%s]: error %d (%s)\n",
+                      bindid ? bindid : "(anon)",
+                      mech, /* mech cannot be SIMPLE here */
+                      rc, ldap_err2string(rc));
+#endif
+	}
+#endif /* SASL_AUTH_SUPPORTED */
+  }
+
+done:
+  ber_bvfree(servercredp);
+  ldap_msgfree(result);
+
+  return rc;
+}
+
+/*
+  emacs settings
+  Local Variables:
+  c-basic-offset: 2
+  End:
+*/
