@@ -33,7 +33,6 @@
 #include "libadminutil/distadm.h"
 #include "libadminutil/srvutil.h"
 #include "dbtadmutil.h"
-#include <ldap_ssl.h>
 
 #ifdef XP_WIN32
 #define strcasecmp stricmp
@@ -92,22 +91,6 @@ find_file_in_paths(
 	return retval;
 }
 
-static int LDAP_CALL LDAP_CALLBACK
-pset_ldap_rebind_proc (LDAP *ld, char **whop, char **passwdp,
-                       int *authmethodp, int freeit, void *arg)
-{
-  PsetPtr pset = (PsetPtr)arg;
-
-  if (freeit == 0) {
-    *whop = pset->binddn;
-    *passwdp = pset->bindpw;
-    *authmethodp = LDAP_AUTH_SIMPLE;
-  }
-
-  return LDAP_SUCCESS;
-}
-
-
 /*
  * PsetNode related data structure and methods
  */
@@ -166,7 +149,7 @@ psetDeletePtr(PsetPtr psetp)
     if (psetp->info) psetNodeDestroy(psetp->info);
     if (psetp->ldapFilter) PR_Free(psetp->ldapFilter);
     if (psetp->ldunbindf) {
-      if (psetp->ld) ldap_unbind(psetp->ld);
+      if (psetp->ld) ldap_unbind_ext(psetp->ld, NULL, NULL);
     }
     if (psetp->configFile) PR_Free(psetp->configFile);
     if (psetp->sieDN) PR_Free(psetp->sieDN);
@@ -199,9 +182,10 @@ psetNodeLDAPDestroy(PsetNodePtr target, LDAP *ld)
   if (ld) {
     nodeDN = ldap_get_dn(ld, target->attrLDAP);
     if (nodeDN) {
-      if ( (ldaperror = ldap_delete_s(ld, nodeDN)) != LDAP_SUCCESS ) {
+      if ( (ldaperror = ldap_delete_ext_s(ld, nodeDN, NULL, NULL)) != LDAP_SUCCESS ) {
 #ifdef LDAP_DEBUG
-        ldap_perror( ld, "ldap_delete_s" );
+        fprintf(stderr, "ldap_delete_s [%s] error %d:%s\n",
+                nodeDN, ldaperror, ldap_err2string(ldaperror));
 #endif
         ldap_memfree(nodeDN);
         /* Well, the destructor CAN NOT return status  */
@@ -399,7 +383,7 @@ psetNodeFindNode(PsetNodePtr nodePtr, LDAP *ld, char *name, int* nodeFlag,
   int         nameLen;
   PsetNodePtr result;
   char        *attrName;
-  char        **vals;
+  struct berval        **vals;
 
    *errorcode = PSET_OP_OK;
    *nodeFlag = 1;
@@ -442,8 +426,8 @@ psetNodeFindNode(PsetNodePtr nodePtr, LDAP *ld, char *name, int* nodeFlag,
     attrName = name+nodeNameLen;
     if (*attrName == '.') attrName++;
     if (nodePtr->attrLDAP) {
-      if ((vals = ldap_get_values(ld, nodePtr->attrLDAP, attrName)) != NULL ) {
-        ldap_value_free(vals);
+      if ((vals = ldap_get_values_len(ld, nodePtr->attrLDAP, attrName)) != NULL ) {
+        ldap_value_free_len(vals);
         *nodeFlag = 0;
         return nodePtr;
       }
@@ -466,15 +450,15 @@ psetNodeFindNode(PsetNodePtr nodePtr, LDAP *ld, char *name, int* nodeFlag,
 ValueType
 psetNodeFindValue(PsetNodePtr nodePtr, LDAP *ld, char *name, int* errorcode)
 {
-  char          **vals;
+  struct berval          **vals;
   ValueType     val;
 
   *errorcode = PSET_OP_OK;
 
   if (nodePtr->attrLDAP) {
-    if (( vals = ldap_get_values(ld, nodePtr->attrLDAP, name )) != NULL ) {
-      val = admutil_strsdup(vals);
-      ldap_value_free(vals);
+    if (( vals = ldap_get_values_len(ld, nodePtr->attrLDAP, name )) != NULL ) {
+      val = admutil_strsdup_berval(vals);
+      ldap_value_free_len(vals);
       return val;
     }
     /* ldap value not available  */
@@ -482,9 +466,9 @@ psetNodeFindValue(PsetNodePtr nodePtr, LDAP *ld, char *name, int* errorcode)
     return NULL;
   }
   else if (nodePtr->attrFile) {
-    vals = treeFindValue(nodePtr->attrFile, name);
+    val = treeFindValue(nodePtr->attrFile, name);
 
-    if (vals) return vals;
+    if (val) return val;
 
     /* No value available in local cache */
     *errorcode = PSET_NO_VALUE;
@@ -573,7 +557,7 @@ psetNodeGetAll(PsetNodePtr psetNode, LDAP *ld, int deep, int* errorcode)
 {
   BerElement          *ber;
   NameType            attrName;
-  char                **vals;
+  struct berval       **vals;
   ListNodePtr         resultList=NULL, tmpList, node;
   char                wholeName[PATH_MAX];
 
@@ -590,16 +574,16 @@ psetNodeGetAll(PsetNodePtr psetNode, LDAP *ld, int deep, int* errorcode)
           strcmp( attrName, "createtimestamp") &&
           strcmp( attrName, "modifytimestamp") &&
           strcmp( attrName, "modifiersname" )) {
-        if ((vals = ldap_get_values(ld, psetNode->attrLDAP, attrName)) !=
+        if ((vals = ldap_get_values_len(ld, psetNode->attrLDAP, attrName)) !=
             NULL) {
           if (psetNode->attrName[0] == '\0')
             PR_snprintf(wholeName, sizeof(wholeName), "%s", attrName);
           else
             PR_snprintf(wholeName, sizeof(wholeName), "%s.%s",
                         psetNode->attrName, attrName);
-          if (vals[0] && *(vals[0]) != '\0') {
+          if (vals[0] && vals[0]->bv_len && vals[0]->bv_val && *(vals[0]->bv_val) != '\0') {
             node = createListNode(wholeName,
-                                  (void*)(admutil_strsdup(vals)),
+                                  (void*)(admutil_strsdup_berval(vals)),
                                   0);
           }
           else {
@@ -607,7 +591,7 @@ psetNodeGetAll(PsetNodePtr psetNode, LDAP *ld, int deep, int* errorcode)
           }
           if (!resultList) resultList = node;
           else listCat(resultList, node);
-          ldap_value_free( vals );
+          ldap_value_free_len( vals );
         }
       }
       ldap_memfree(attrName);
@@ -639,7 +623,7 @@ psetNodeGetAllACI(PsetNodePtr psetNode, LDAP *ld, int deep, int* errorcode)
 {
   BerElement          *ber;
   NameType            attrName;
-  char                **vals;
+  struct berval       **vals;
   ListNodePtr         resultList=NULL, tmpList, node;
   char                wholeName[PATH_MAX];
 
@@ -655,16 +639,16 @@ psetNodeGetAllACI(PsetNodePtr psetNode, LDAP *ld, int deep, int* errorcode)
           strcmp( attrName, "createtimestamp") &&
           strcmp( attrName, "modifytimestamp") &&
           strcmp( attrName, "modifiersname" )) {
-        if ((vals = ldap_get_values(ld, psetNode->attrLDAP, attrName)) !=
+        if ((vals = ldap_get_values_len(ld, psetNode->attrLDAP, attrName)) !=
             NULL) {
           if (psetNode->attrName[0] == '\0')
             PR_snprintf(wholeName, sizeof(wholeName), "%s", attrName);
           else
             PR_snprintf(wholeName, sizeof(wholeName), "%s.%s",
                         psetNode->attrName, attrName);
-          if (vals[0] && *(vals[0]) != '\0') {
+          if (vals[0] && vals[0]->bv_len && vals[0]->bv_val && *(vals[0]->bv_val) != '\0') {
             node = createListNode(wholeName,
-                                  (void*)(admutil_strsdup(vals)),
+                                  (void*)(admutil_strsdup_berval(vals)),
                                   0);
           }
           else {
@@ -672,7 +656,7 @@ psetNodeGetAllACI(PsetNodePtr psetNode, LDAP *ld, int deep, int* errorcode)
           }
           if (!resultList) resultList = node;
           else listCat(resultList, node);
-          ldap_value_free( vals );
+          ldap_value_free_len( vals );
         }
       }
       ldap_memfree(attrName);
@@ -711,9 +695,10 @@ psetNodeLDAPUpdate(PsetNodePtr nodePtr, LDAP *ld, char* ldFilter, LDAPMod **mods
   if (!nodeDN) return PSET_OP_FAIL;
 
 
-  if ( (ldaperror = ldap_modify_s(ld, nodeDN, mods)) != LDAP_SUCCESS ) {
+  if ( (ldaperror = ldap_modify_ext_s(ld, nodeDN, mods, NULL, NULL)) != LDAP_SUCCESS ) {
 #ifdef LDAP_DEBUG
-    ldap_perror( ld, "ldap_modify_s" );
+    fprintf(stderr, "ldap_modify_s: [%s] error %d:%s\n",
+            nodeDN, ldaperror, ldap_err2string(ldaperror));
 #endif
     ldap_memfree(nodeDN);
     if (ldaperror == LDAP_INSUFFICIENT_ACCESS) return PSET_ACCESS_FAIL;
@@ -724,11 +709,12 @@ psetNodeLDAPUpdate(PsetNodePtr nodePtr, LDAP *ld, char* ldFilter, LDAPMod **mods
 
   if (ldFilter) filter = ldFilter;
   else filter = "(objectclass=*)";
-  if ((ldaperror = ldap_search_s(ld, nodeDN, LDAP_SCOPE_BASE,
-                                 filter, NULL, 0, &result)) 
+  if ((ldaperror = ldap_search_ext_s(ld, nodeDN, LDAP_SCOPE_BASE,
+                                     filter, NULL, 0, NULL, NULL, NULL, -1, &result)) 
        != LDAP_SUCCESS ) {
 #ifdef LDAP_DEBUG
-    ldap_perror(ld, "ldap_search_s" );
+    fprintf(stderr, "ldap_search_s: [%s:%s] error %d:%s\n",
+            nodeDN, filter, ldaperror, ldap_err2string(ldaperror));
 #endif
     ldap_memfree(nodeDN);
     if (ldaperror == LDAP_INSUFFICIENT_ACCESS) return PSET_ACCESS_FAIL;
@@ -771,6 +757,10 @@ psetNodeFileExport(PsetNodePtr nodePtr, LDAP *ld, FILE *fstream)
             valBuf = (char *)PR_Realloc(valBuf, valLen + 1); /*+1 for ending '\0'*/
             prevValLen = valLen;
           }
+          if (!valBuf) {
+            errorCode = PSET_SYSTEM_ERR;
+            goto bail;
+          }
           PR_snprintf(valBuf, valLen + 1, "%s", valList[i]);
           sptr = valBuf;
           while ((cptr = strchr(sptr, '\n'))) {
@@ -784,6 +774,7 @@ psetNodeFileExport(PsetNodePtr nodePtr, LDAP *ld, FILE *fstream)
       }
       listPtr = listPtr->next;
     }
+bail:
     nvlistDestroy(resultList);
   }
 
@@ -812,11 +803,12 @@ psetLDAPRefresh(PsetPtr pset)
   if (pset->ldapFilter) filter = pset->ldapFilter;
   else filter = "(objectclass=*)";
 
-  if ((ldaperror = ldap_search_s(pset->ld, pset->sieDN, LDAP_SCOPE_SUBTREE,
-                                 filter, NULL, 0, &result)) 
+  if ((ldaperror = ldap_search_ext_s(pset->ld, pset->sieDN, LDAP_SCOPE_SUBTREE,
+                                     filter, NULL, 0, NULL, NULL, NULL, -1, &result)) 
        != LDAP_SUCCESS ) {
 #ifdef LDAP_DEBUG
-    ldap_perror( pset->ld, "ldap_search_s" );
+    fprintf(stderr, "ldap_search_s: [%s:%s] error %d:%s\n",
+            pset->sieDN, filter, ldaperror, ldap_err2string(ldaperror));
 #endif
     ldap_msgfree(result);
     if (ldaperror == LDAP_INSUFFICIENT_ACCESS) return PSET_ACCESS_FAIL;
@@ -835,11 +827,10 @@ psetLDAPRefresh(PsetPtr pset)
     nodeName = dn2AttrName(dn, pset->sieDN);
     ldap_memfree(dn);
 
-    if (nodeName[0] == '\0') {
+    if (!nodeName || (nodeName[0] == '\0')) {
       psetNodeSetLDAP(pset->info, result);
       pset->info->attrLDAP = e;
-    }
-    else {
+    } else {
       target = psetNodeFindNode(pset->info, pset->ld, nodeName, &nodeFlag,
                                 &errorCode);
       if (target) {
@@ -922,12 +913,14 @@ psetFileImportP(PsetPtr pset)
     case 1:
       /* EOF, out of here */
       if (namebuf[0] != '\0') {
-        if (nodeName[0] == '\0') target = pset->info;
-        else if (lastPsetNode && !strcasecmp(nodeName, lastPsetNode->attrName))
+        if (!nodeName || (nodeName[0] == '\0')) {
+          target = pset->info;
+        } else if (lastPsetNode && !strcasecmp(nodeName, lastPsetNode->attrName)) {
           /* Is the last node? -> strcasecmp node name */
           target = lastPsetNode;
-        else   /* No, Locate and build node -> psetnodeBuildNodes  */
+        } else {  /* No, Locate and build node -> psetnodeBuildNodes  */
           target = psetNodeBuildNodes(pset->info, nodeName, &errorCode);
+        }
 
         /* psetNodeAddNameValue  */
         if (target) {
@@ -971,12 +964,14 @@ psetFileImportP(PsetPtr pset)
       }
       else {      
         if (namebuf[0] != '\0') {
-          if (nodeName[0] == '\0') target = pset->info;
-          else if (lastPsetNode && !strcasecmp(nodeName, lastPsetNode->attrName))
+          if (!nodeName || (nodeName[0] == '\0')) {
+            target = pset->info;
+          } else if (lastPsetNode && !strcasecmp(nodeName, lastPsetNode->attrName)) {
             /* Is the last node? -> strcasecmp node name */
             target = lastPsetNode;
-          else   /* No, Locate and build node -> psetnodeBuildNodes  */
+          } else {  /* No, Locate and build node -> psetnodeBuildNodes  */
             target = psetNodeBuildNodes(pset->info, nodeName, &errorCode);
+          }
 
           /* psetNodeAddNameValue  */
           if (target) {
@@ -1200,7 +1195,6 @@ psetRootModAttrList(PsetRootPtr psetRoot, int mode, AttributeList nvl)
   PsetNodePtr   target;
   ListNodePtr   updateList = NULL, attrList, nodePtr, attrPtr;
   int           nodeFlag;
-  int           partial = 0;
   ValueType     val;
 
   updateList = createUpdateList(nvl);
@@ -1212,11 +1206,10 @@ psetRootModAttrList(PsetRootPtr psetRoot, int mode, AttributeList nvl)
         (pset->configFile) && 
         !(pset->fileRW)) {
       destroyUpdateList(updateList);
-      if (partial) return PSET_PARTIAL_OP;
-      else return PSET_LOCAL_MODE;
+      return PSET_LOCAL_MODE;
     }
 
-    if (target && nodeFlag) {
+    if (target && nodeFlag && pset) {
       attrList = (ListNodePtr)nodePtr->val;
 
       /* Check the existence of the attribute */
@@ -1230,8 +1223,7 @@ psetRootModAttrList(PsetRootPtr psetRoot, int mode, AttributeList nvl)
           if (!val) {
             /* Arrtibute does not exist */
             destroyUpdateList(updateList);
-            if (partial) return PSET_PARTIAL_OP;
-            else return PSET_ATTR_NOT_EXIST;
+            return PSET_ATTR_NOT_EXIST;
           }
           else deleteValue (val);
           break;
@@ -1240,8 +1232,7 @@ psetRootModAttrList(PsetRootPtr psetRoot, int mode, AttributeList nvl)
             /* Attribute already exist */
             deleteValue (val);
             destroyUpdateList(updateList);
-            if (partial) return PSET_PARTIAL_OP;
-            else return PSET_ATTR_EXIST;
+            return PSET_ATTR_EXIST;
           }
           break;
         default:
@@ -1292,13 +1283,14 @@ psetRootModAttrList(PsetRootPtr psetRoot, int mode, AttributeList nvl)
       errorcode = PSET_NOT_ENTRY;
       destroyUpdateList(updateList);
       if (errorcode) {
-        if (partial) return PSET_PARTIAL_OP;
-        else return errorcode;
+        return errorcode;
       }
     }
     nodePtr=nodePtr->next;
   }
-  psetFileExportP(pset);
+  if (pset) {
+    psetFileExportP(pset);
+  }
   destroyUpdateList(updateList);
   return errorcode;
 }
@@ -1380,7 +1372,7 @@ psetCreate(char* serverID, char* configRoot, char* user, char* passwd,
       *errorcode = PSET_ENV_ERR;
   }
 
-  pset = psetRealCreate(ldapHost, ldapPort, sieDN, userDN, bindPasswd, path,
+  pset = psetRealCreate(ldapInfo, ldapHost, ldapPort, sieDN, userDN, bindPasswd, path,
                         errorcode);
 done:
   PR_Free(ldapHost);
@@ -1400,11 +1392,11 @@ done:
 
 
 PR_IMPLEMENT(PsetHndl)
-psetRealCreate(char* ldapHost, int ldapPort, char* sieDN, char* userDN,
+psetRealCreate(AdmldapInfo ldapInfo, char* ldapHost, int ldapPort, char* sieDN, char* userDN,
                char* passwd, char* configFile, int* errorcode)
 {
 
-  return psetRealLDAPImport(NULL, ldapHost, ldapPort, sieDN, userDN,
+  return psetRealLDAPImport(ldapInfo, NULL, ldapHost, ldapPort, sieDN, userDN,
                             passwd, configFile, NULL, errorcode);
 }
 
@@ -1432,7 +1424,7 @@ psetFileCreate(char* configFile, char* filter, int* errorcode)
 }
 
 PR_IMPLEMENT(PsetHndl)
-psetRealLDAPImport(PsetHndl pseth, char* ldapHost, int ldapPort, char* sieDN, 
+psetRealLDAPImport(AdmldapInfo ldapInfo, PsetHndl pseth, char* ldapHost, int ldapPort, char* sieDN, 
                    char* userDN, char* passwd, char* cacheFile, char* filter,
                    int* errorcode)
 {
@@ -1447,9 +1439,9 @@ psetRealLDAPImport(PsetHndl pseth, char* ldapHost, int ldapPort, char* sieDN,
   }
 
   if (NULL != passwd) {
-    ld = ldap_init(ldapHost, ldapPort);
+    ld = admutil_ldap_init(ldapInfo, NULL, ldapHost, ldapPort, 0, 0, NULL);
     /* authenticate to LDAP server*/
-    if ((ldapError = ldap_simple_bind_s(ld, userDN, passwd))
+    if ((ldapError = admutil_ldap_bind(ld, userDN, passwd, LDAP_SASL_SIMPLE, NULL, NULL, NULL, NULL))
         != LDAP_SUCCESS ) {
       switch (ldapError) {
       case LDAP_INAPPROPRIATE_AUTH:
@@ -1457,20 +1449,21 @@ psetRealLDAPImport(PsetHndl pseth, char* ldapHost, int ldapPort, char* sieDN,
       case LDAP_INSUFFICIENT_ACCESS:
         /* authenticate failed: Should not continue */
   #ifdef LDAP_DEBUG
-        ldap_perror( ld, "ldap_simple_bind_s" );
+    fprintf(stderr, "ldap_bind: [%s] error %d:%s\n",
+            userDN, ldapError, ldap_err2string(ldapError));
   #endif
-        ldap_unbind(ld);
+        ldap_unbind_ext(ld, NULL, NULL);
         *errorcode = PSET_AUTH_FAIL;
         return pset;
       case LDAP_NO_SUCH_OBJECT:
       case LDAP_ALIAS_PROBLEM:
       case LDAP_INVALID_DN_SYNTAX:
         /* Not a good DN */
-        ldap_unbind(ld);
+        ldap_unbind_ext(ld, NULL, NULL);
         *errorcode = PSET_ENTRY_NOT_EXIST;
         return pset;
       default:
-        ldap_unbind(ld);
+        ldap_unbind_ext(ld, NULL, NULL);
         unbindF = 0;
         ld = NULL;
       }
@@ -2054,6 +2047,8 @@ psetAddEntry(PsetHndl pseth, NameType parent, NameType name,
   errorCode = PSET_OP_OK;
 
   if (pset->ld) {
+    int adderr;
+
     dn = attrName2dn(absAttrName, pset->sieDN);
 
     /* Build initial list */
@@ -2084,15 +2079,16 @@ psetAddEntry(PsetHndl pseth, NameType parent, NameType name,
       mods[i++] = createMod(nv->attrName, nv->attrVal, 0);
     mods[i] = NULL;
 
-    if ( ldap_add_s (pset->ld, dn, mods) == LDAP_SUCCESS ) {
+    if ( (adderr = ldap_add_ext_s (pset->ld, dn, mods, NULL, NULL)) == LDAP_SUCCESS ) {
       /* Refresh this node  */
       if (pset->ldapFilter) filter = pset->ldapFilter;
       else filter = "(objectclass=*)";
-      if ((ldaperror = ldap_search_s(pset->ld, dn, LDAP_SCOPE_BASE,
-                                     filter, NULL, 0, &result)) 
+      if ((ldaperror = ldap_search_ext_s(pset->ld, dn, LDAP_SCOPE_BASE,
+                                         filter, NULL, 0, NULL, NULL, NULL, -1, &result)) 
           != LDAP_SUCCESS ) {
 #ifdef LDAP_DEBUG
-        ldap_perror(pset->ld, "ldap_search_s" );
+        fprintf(stderr, "ldap_search_s: [%s:%s] error %d:%s\n",
+                dn, filter, ldaperror, ldap_err2string(ldaperror));
 #endif
         if (ldaperror == LDAP_INSUFFICIENT_ACCESS) return PSET_ACCESS_FAIL;
         return PSET_SYSTEM_ERR;
@@ -2112,7 +2108,8 @@ psetAddEntry(PsetHndl pseth, NameType parent, NameType name,
     }
     else {
 #ifdef LDAP_DEBUG
-      ldap_perror(pset->ld, "ldap_add_s");
+      fprintf(stderr, "ldap_add_s: [%s] error %d:%s\n",
+              dn, adderr, ldap_err2string(adderr));
 #endif
       errorCode = PSET_OP_FAIL;
     }
@@ -2359,6 +2356,34 @@ psetErrorString(int errorNum, char* lang, char *buffer, size_t bufsize, int *rc)
   return PL_strdup(errorStr);
 }
 
+#if defined(USE_OPENLDAP)
+static int
+pset_ldap_rebind_proc(
+	LDAP *ld, LDAP_CONST char *url,
+	ber_tag_t request, ber_int_t msgid,
+	void *arg)
+{
+  PsetPtr pset = (PsetPtr)arg;
+
+  return admutil_ldap_bind(ld, pset->binddn, pset->bindpw, LDAP_SASL_SIMPLE, NULL, NULL, NULL, NULL);
+}
+#else /* NOT OPENLDAP */
+static int
+pset_ldap_rebind_proc (LDAP *ld, char **whop, char **passwdp,
+                       int *authmethodp, int freeit, void *arg)
+{
+  PsetPtr pset = (PsetPtr)arg;
+
+  if (freeit == 0) {
+    *whop = pset->binddn;
+    *passwdp = pset->bindpw;
+    *authmethodp = LDAP_AUTH_SIMPLE;
+  }
+
+  return LDAP_SUCCESS;
+}
+#endif /* OPENLDAP */
+
 /* Setting  up LDAP referal */
 PR_IMPLEMENT(int)
 psetSetLDAPReferalInfo(PsetHndl pseth, char* userDN, char* passwd)
@@ -2618,3 +2643,10 @@ psetReplicateSIE(PsetHndl source, PsetHndl dest, char *source_groupDN, char *des
 
   return rv;
 }
+
+/*
+  emacs settings
+  Local Variables:
+  c-basic-offset: 2
+  End:
+*/
