@@ -23,7 +23,7 @@
 #include <stdlib.h>
 #include <ctype.h>
 #include <ldap.h>
-#include "admutil_pvt.h"
+#include "libadminutil/admutil-int.h"
 #include "libadminutil/psetc.h"
 
 #ifdef XP_WIN32
@@ -36,9 +36,8 @@ admldapCreateLDAPHndl(AdmldapInfo info, char *targetDN, int *error_code)
 {
   char            *sieDN = NULL;
   char            *resultDN = NULL;
-  AdmldapHdnlPtr  hndl = (AdmldapHdnlPtr)info;
 
-  if (!hndl) { *error_code = UG_EMPTY_LDAPINFO; return NULL; }
+  if (!info) { *error_code = UG_EMPTY_LDAPINFO; return NULL; }
   *error_code = UG_OP_OK;
 
   if (targetDN) resultDN = PL_strdup(targetDN);
@@ -46,7 +45,7 @@ admldapCreateLDAPHndl(AdmldapInfo info, char *targetDN, int *error_code)
     /* No target DN given, try to figure it out */
     if (!sieDN) sieDN = admldapGetSIEDN(info);
     if (sieDN) {
-      resultDN = PR_smprintf("cn=configuration, %s", sieDN);
+      resultDN = PR_smprintf("cn=configuration,%s", sieDN);
     }
     else { 
       *error_code = UG_NO_TARGETDN; 
@@ -61,12 +60,12 @@ admldapGetUserDirectoryInfo(LDAP *ld, char* targetDN, char** directoryURL,
                         char** bindDN, char** bindPassword,
                         char** directoryInfoRef, int* error_code)
 {
-  int         ldaperror;
-  LDAPMessage *e, *result;
-  char        **directoryURLVals = NULL, 
-              **bindDNVals = NULL,
-              **bindPasswordVals = NULL,
-              **directoryInfoRefVals = NULL;
+  int           ldaperror;
+  LDAPMessage   *e, *result;
+  struct berval **directoryURLVals = NULL, 
+                **bindDNVals = NULL,
+                **bindPasswordVals = NULL,
+                **directoryInfoRefVals = NULL;
 
   *error_code = UG_OP_OK;
   *directoryURL = NULL;
@@ -77,17 +76,18 @@ admldapGetUserDirectoryInfo(LDAP *ld, char* targetDN, char** directoryURL,
   if (!ld) { *error_code = UG_NULL_LDAP; return 0; }
   if (!targetDN)  { *error_code = UG_NULL_DN; return 0; }
 
-  if ((ldaperror = ldap_search_s(ld, 
-                                 targetDN, 
-                                 LDAP_SCOPE_BASE,
-                                 "(objectclass=nsDirectoryInfo)", 
-                                 NULL, 
-                                 0, 
-                                 &result)) 
+  if ((ldaperror = ldap_search_ext_s(ld, 
+                                     targetDN, 
+                                     LDAP_SCOPE_BASE,
+                                     "(objectclass=nsDirectoryInfo)", 
+                                     NULL, 
+                                     0, NULL, NULL, NULL, -1,
+                                     &result))
       != LDAP_SUCCESS ) {
     ldap_msgfree(result);
 #ifdef LDAP_DEBUG
-    ldap_perror(ld, "ldap_search_s");
+    fprintf(stderr, "ldap_search_s: [%s:%s] error %d:%s\n",
+            targetDN, "(objectclass=nsDirectoryInfo)", ldaperror, ldap_err2string(ldaperror));
 #endif
     if (ldaperror == LDAP_INSUFFICIENT_ACCESS ||
         ldaperror == LDAP_INAPPROPRIATE_AUTH) {
@@ -110,48 +110,51 @@ admldapGetUserDirectoryInfo(LDAP *ld, char* targetDN, char** directoryURL,
   }
 
   e = ldap_first_entry(ld, result);
-  directoryURLVals = ldap_get_values(ld, e, "nsDirectoryURL");
-  bindDNVals = ldap_get_values(ld, e, "nsBindDN");
-  bindPasswordVals = ldap_get_values(ld, e, "nsBindPassword");
-  directoryInfoRefVals = ldap_get_values(ld, e, "nsDirectoryInfoRef");
+  directoryURLVals = ldap_get_values_len(ld, e, "nsDirectoryURL");
+  bindDNVals = ldap_get_values_len(ld, e, "nsBindDN");
+  bindPasswordVals = ldap_get_values_len(ld, e, "nsBindPassword");
+  directoryInfoRefVals = ldap_get_values_len(ld, e, "nsDirectoryInfoRef");
 
-  if (directoryURLVals) {
-    char **s;
+  if (directoryURLVals && directoryURLVals[0]) {
+    struct berval **s;
     char buffer[1024];
     
-    s = ldap_get_values(ld, e, "nsDirectoryFailoverList");
-    if (s && strcmp(s[0], "")) {
-      char *temp = strrchr(directoryURLVals[0], '/');
+    s = ldap_get_values_len(ld, e, "nsDirectoryFailoverList");
+    if (s && s[0] && s[0]->bv_len && s[0]->bv_val && strcmp(s[0]->bv_val, "")) {
+      char *temp = PL_strnchr(directoryURLVals[0]->bv_val, '/', directoryURLVals[0]->bv_len);
       /* append failover list to url */
       if (NULL != temp) {
         *temp = '\0';
-        PR_snprintf(buffer, sizeof(buffer), "%s %s/%s",
-                    directoryURLVals[0], s[0], temp + 1);
+        PR_snprintf(buffer, sizeof(buffer), "%.*s %.*s/%s",
+                    directoryURLVals[0]->bv_len, directoryURLVals[0]->bv_val,
+                    s[0]->bv_len, s[0]->bv_val, temp + 1);
       } else {
-        PR_snprintf(buffer, sizeof(buffer), "%s %s", directoryURLVals[0], s[0]);
+        PR_snprintf(buffer, sizeof(buffer), "%.*s %.*s",
+                    directoryURLVals[0]->bv_len, directoryURLVals[0]->bv_val,
+                    s[0]->bv_len, s[0]->bv_val);
       }
     } else {
-      PR_snprintf(buffer, sizeof(buffer), "%s", directoryURLVals[0]);
+      PR_snprintf(buffer, sizeof(buffer), "%.*s",
+                  directoryURLVals[0]->bv_len, directoryURLVals[0]->bv_val);
     }
-    ldap_value_free(s);
+    ldap_value_free_len(s);
     
     *directoryURL = PL_strdup(buffer);
-    ldap_value_free(directoryURLVals);
+    ldap_value_free_len(directoryURLVals);
   }
-  if (bindDNVals) {
-    *bindDN = PL_strdup(bindDNVals[0]);
-    ldap_value_free(bindDNVals);
+  if (bindDNVals && bindDNVals[0]) {
+    *bindDN = PL_strndup(bindDNVals[0]->bv_val, bindDNVals[0]->bv_len);
+    ldap_value_free_len(bindDNVals);
   }
-  if (bindPasswordVals) {
-    *bindPassword = PL_strdup(bindPasswordVals[0]);
-    if (bindPasswordVals[0]) {
-      memset(bindPasswordVals[0], '\0', strlen(bindPasswordVals[0]));
-    }
-    ldap_value_free(bindPasswordVals);
+  if (bindPasswordVals && bindPasswordVals[0]) {
+    *bindPassword = PL_strndup(bindPasswordVals[0]->bv_val, bindPasswordVals[0]->bv_len);
+    /* clear out this memory */
+    memset(bindPasswordVals[0]->bv_val, '\0', bindPasswordVals[0]->bv_len);
+    ldap_value_free_len(bindPasswordVals);
   }
-  if (directoryInfoRefVals) {
-    *directoryInfoRef = PL_strdup(directoryInfoRefVals[0]);
-    ldap_value_free(directoryInfoRefVals);
+  if (directoryInfoRefVals && directoryInfoRefVals[0]) {
+    *directoryInfoRef = PL_strndup(directoryInfoRefVals[0]->bv_val, directoryInfoRefVals[0]->bv_len);
+    ldap_value_free_len(directoryInfoRefVals);
   }
   
   ldap_msgfree(result);
@@ -176,7 +179,7 @@ admldapGetAdmGroupUGDN(char* sieDN)
               dnList[dnLen-2],
               dnList[dnLen-1]);
   }
-  ldap_value_free(dnList);
+  admutil_strsFree(dnList);
   return dnbuf;
 }
 
@@ -191,11 +194,11 @@ admldapGetDomainUGDN(char* sieDN)
   while (dnList && dnList[dnLen]) dnLen++;
 
   if (dnLen >= 3) {
-    dnbuf = PR_smprintf("cn=UserDirectory, ou=Global Preferences,%s,%s", 
+    dnbuf = PR_smprintf("cn=UserDirectory,ou=Global Preferences,%s,%s", 
               dnList[dnLen-2],
               dnList[dnLen-1]);
   }
-  ldap_value_free(dnList);
+  admutil_strsFree(dnList);
 
   return dnbuf;
 }
@@ -442,9 +445,10 @@ admldapSetUserDirectoryReal(LDAP *ld, char* targetDN, char* directoryURL,
     mods[i] = NULL;
   }
 
-  if ( (ldaperror = ldap_modify_s(ld, targetDN, mods)) != LDAP_SUCCESS ) {
+  if ( (ldaperror = ldap_modify_ext_s(ld, targetDN, mods, NULL, NULL)) != LDAP_SUCCESS ) {
 #ifdef LDAP_DEBUG
-    ldap_perror(ld, "ldap_modify_s");
+    fprintf(stderr, "ldap_modify_s: [%s] error %d:%s\n",
+            targetDN, ldaperror, ldap_err2string(ldaperror));
 #endif    
     if (ldaperror == LDAP_INSUFFICIENT_ACCESS) *error_code = UG_ACCESS_FAIL;
     else *error_code = UG_LDAP_SYSTEM_ERR;
@@ -480,13 +484,12 @@ admldapGetUserDirectory(AdmldapInfo info, char* targetDN, char** directoryURL,
 {
   int             status = 1;
   char            *realTargetDN;
-  AdmldapHdnlPtr  hndl = (AdmldapHdnlPtr)info;
 
   realTargetDN = admldapCreateLDAPHndl(info, targetDN, error_code);
 
   if (!realTargetDN) return 0;
 
-  status = admldapGetUserDirectoryReal(hndl->ldapHndl,
+  status = admldapGetUserDirectoryReal(admldapGetLDAPHndl(info),
                                        realTargetDN,
                                        directoryURL,
                                        bindDN,
@@ -505,13 +508,12 @@ admldapSetUserDirectory(AdmldapInfo info, char* targetDN, char* directoryURL,
 {
   int             status = 1;
   char            *realTargetDN;
-  AdmldapHdnlPtr  hndl = (AdmldapHdnlPtr)info;
 
   realTargetDN = admldapCreateLDAPHndl(info, targetDN, error_code);
 
   if (!realTargetDN) return 0;
 
-  status = admldapSetUserDirectoryReal(hndl->ldapHndl,
+  status = admldapSetUserDirectoryReal(admldapGetLDAPHndl(info),
                                        realTargetDN,
                                        directoryURL,
                                        bindDN,
